package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.entity.ConfigShowEnt;
import pps.core.prediction.entity.PlantPredictedPowerShortTermEnt;
import pps.core.prediction.entity.ThirdActivePowerView;
import pps.core.prediction.mapper.ConfigShowMapper;
import pps.core.prediction.mapper.PlantPredictedPowerShortTermMapper;
import pps.core.prediction.mapper.ThirdActivePowerViewMapper;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPredictedPowerOutput;
import pps.core.prediction.service.data.plant_predicted_power_short_term.GetPlantPredictedPowerShortTermInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 短期功率预测
 *
 * @author ZWT
 * @date 2024/06/06
 */
@XService
@XText("电站预测功率短期72小时数据")
public class PlantPredictedPowerShortTermService {

    /**
     * 三天功率预测
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPredictedPowerOutput }>
     */
    @XApiGet
    @XText("根据电站预测3天的预测数据")
    public XListResult<GetPredictedPowerOutput> getThreePlantPredictedPowerShortTermList(XContext context, GetPlantPredictedPowerShortTermInput input) {
        return XListResult.success(this.getPredictedPowerOutput(context, input.getStartTime(), input.getPlantId(), 3));
    }

    /**
     * 一天功率预测
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPredictedPowerOutput }>
     */
    @XApiGet
    @XText("根据电站预测1天的预测数据")
    public XListResult<GetPredictedPowerOutput> getOnePlantPredictedPowerShortTermList(XContext context, GetPlantPredictedPowerShortTermInput input) {
        return XListResult.success(this.getPredictedPowerOutput(context, input.getStartTime(), input.getPlantId(), 1));
    }

    /**
     * 获得预测功率输出
     *
     * @param context      上下文
     * @param inputTimeStr 输入时间str
     * @param stationId    工作站id
     * @param offsetDay    抵消日
     * @return {@link List }<{@link GetPredictedPowerOutput }>
     */
    private List<GetPredictedPowerOutput> getPredictedPowerOutput(XContext context, String inputTimeStr, String stationId, int offsetDay) {
        //判断是否需要启用演示配置
        DateTime date = DateUtil.date();
        DateTime inputTime = DateUtil.parse(inputTimeStr, BusinessConstant.DATE_FORMAT_DAY);
        ConfigShowMapper configMapper = context.getBean(ConfigShowMapper.class);
        ConfigShowEnt configShowEnt = configMapper.selectOne(new LambdaQueryWrapper<ConfigShowEnt>()
                .last("LIMIT 1"));
        Date showTime;
        DateTime day;
        int betweenDay;
        if (ObjectUtil.isAllNotEmpty(configShowEnt, configShowEnt.getIsEnable(), configShowEnt.getChoiceDate())) {
            day = configShowEnt.getIsEnable().equals(0) ? DateUtil.date(configShowEnt.getChoiceDate()) : inputTime;
            showTime = DateUtil.parse(day.toString(BusinessConstant.DATE_FORMAT_DAY) + " " + date.toString(BusinessConstant.TIME_FORMAT));
            //计算偏移
            betweenDay = (int) DateUtil.betweenDay(inputTime, day, false);
            if (DateUtil.compare(day, inputTime) > 0) {
                betweenDay = -betweenDay;
            }
        } else {
            day = inputTime;
            showTime = date;
            betweenDay = 0;
        }
        DateTime beginTime = DateUtil.offsetMinute(day, -1);
        DateTime endTime = DateUtil.offsetDay(day, offsetDay);
        //查短期预测
        PlantPredictedPowerShortTermMapper mapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
        List<PlantPredictedPowerShortTermEnt> predictedList = mapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerShortTermEnt>()
                .eq(PlantPredictedPowerShortTermEnt::getPlantId, stationId)
                .between(PlantPredictedPowerShortTermEnt::getDataDate, beginTime, endTime)
                .orderByAsc(PlantPredictedPowerShortTermEnt::getDataDate)
        );
        Map<DateTime, BigDecimal> predictedMap;
        if (CollUtil.isNotEmpty(predictedList)) {
            predictedMap = predictedList.stream()
                    .collect(Collectors.toMap(x -> DateUtil.parse(x.getDataDate()), PlantPredictedPowerShortTermEnt::getPredictPower));
        } else {
            predictedMap = Collections.emptyMap();
        }
        //查实际功率
        ThirdActivePowerViewMapper powerMapper = context.getBean(ThirdActivePowerViewMapper.class);
        List<ThirdActivePowerView> powerList = powerMapper.selectAvgPower15(ThirdActivePowerView.builder()
                .stationId(stationId)
                .startTime(beginTime)
                .endTime(showTime)
                .build());
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(powerList)) {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(ThirdActivePowerView::getInputTime, ThirdActivePowerView::getPhotovoltaicPower));
        } else {
            powerMap = Collections.emptyMap();
        }
        //取时间范围
        List<DateTime> rangeToList = DateUtil.rangeToList(day, endTime, DateField.MINUTE, 15);
        GetPredictedPowerOutput output;
        List<GetPredictedPowerOutput> outputs = new ArrayList<>(rangeToList.size());
        for (DateTime dateTime : rangeToList) {
            output = new GetPredictedPowerOutput();
            //匹配预测发电量
            output.setPredictPower(predictedMap.getOrDefault(dateTime, BigDecimal.ZERO));
            //匹配实际发电量
            output.setPower(powerMap.getOrDefault(dateTime, BigDecimal.ZERO));
            //时间偏移
            if (betweenDay != 0) {
                output.setDataDate(DateUtil.offsetDay(dateTime, betweenDay));
            } else {
                output.setDataDate(dateTime);
            }
            outputs.add(output);
        }
        return outputs;
    }
}